@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright: (c) 2021, Dell EMC
+# Copyright: (c) 2021, Dell Technologies
 
 """Collection of configuration related functions for PowerStore"""
 
@@ -520,6 +520,90 @@ class Configuration:
         )
         return self.get_cluster_details(cluster_id)
 
+    def cluster_create_validate(self, cluster, appliances, dns_servers,
+                                ntp_servers, networks,
+                                is_http_redirect_enabled,
+                                physical_switches=None, vcenters=None):
+        """Validate the creation of cluster configuration
+        :param cluster: Dict containing parameters for the cluster
+        :type cluster: dict
+        :param appliances: List containing appliances
+        :type appliances: list[dict]
+        :param dns_servers: List containing IP addresses for DNS Servers
+        :type dns_servers: list[str]
+        :param ntp_servers: List containing IP addresses for NTP Servers
+        :type ntp_servers: list[dict]
+        :param physical_switches: List containing physical switches
+        :type physical_switches: list[dict]
+        :param networks: List containing networks
+        :type networks: list[dict]
+        :param vcenters: List of vCenters
+        :type vcenters: list[dict]
+        :param is_http_redirect_enabled: Whether to redirect the http requests
+            to https
+        :type is_http_redirect_enabled: bool
+        :return: None
+        :rtype: None
+        """
+        LOG.info("Validating the create cluster configuration")
+        cluster_url = constants.CREATE_CLUSTER_VALIDATE_URL
+
+        cluster_payload = self.\
+            _prepare_create_cluster_payload(is_http_redirect_enabled=is_http_redirect_enabled,
+                                            cluster=cluster,
+                                            appliances=appliances,
+                                            dns_servers=dns_servers,
+                                            ntp_servers=ntp_servers,
+                                            physical_switches=physical_switches,
+                                            networks=networks,
+                                            vcenters=vcenters)
+        return self.config_client.request(constants.POST,
+                                          cluster_url.format(self.server_ip),
+                                          payload=cluster_payload)
+
+    def cluster_create(self, cluster, appliances, dns_servers, ntp_servers,
+                       networks, is_http_redirect_enabled,
+                       physical_switches=None, vcenters=None, is_async=False):
+        """Create a Cluster of one or more appliances
+        :param cluster: Dict containing parameters for the cluster
+        :type cluster: dict
+        :param appliances: List containing appliances
+        :type appliances: list[dict]
+        :param dns_servers: List containing IP addresses for DNS Servers
+        :type dns_servers: list[str]
+        :param ntp_servers: List containing IP addresses for NTP Servers
+        :type ntp_servers: list[dict]
+        :param physical_switches: List containing physical switches
+        :type physical_switches: list[dict]
+        :param networks: List containing networks
+        :type networks: list[dict]
+        :param vcenters: List of vCenters
+        :type vcenters: list[dict]
+        :param is_http_redirect_enabled: Whether to redirect the http requests
+            to https
+        :type is_http_redirect_enabled: bool
+        :param is_async: Flag to indicate sync/async operation
+        :type is_async: bool
+        :return: Unique identifier of the new instance created
+        :rtype: str
+        """
+        LOG.info("Creating new cluster configuration")
+        cluster_url = constants.CREATE_CLUSTER_URL
+        if is_async:
+            cluster_url = cluster_url + "?is_async=true"
+        cluster_payload = self.\
+            _prepare_create_cluster_payload(is_http_redirect_enabled=is_http_redirect_enabled,
+                                            cluster=cluster,
+                                            appliances=appliances,
+                                            dns_servers=dns_servers,
+                                            ntp_servers=ntp_servers,
+                                            physical_switches=physical_switches,
+                                            networks=networks,
+                                            vcenters=vcenters)
+        return self.config_client.request(constants.POST,
+                                          cluster_url.format(self.server_ip),
+                                          payload=cluster_payload)
+
     # Cluster operations end
 
     # CHAP config operations start
@@ -817,7 +901,6 @@ class Configuration:
     # IP ports operations end
 
     # vCenter operations start
-
     def get_vcenters(self, filter_dict=None, all_pages=False):
         """Get all vcenters.
         :param filter_dict: (optional) Filter details
@@ -832,11 +915,18 @@ class Configuration:
         querystring = helpers.prepare_querystring(
             constants.SELECT_ID, filter_dict)
         LOG.info("Querystring: '%s'" % querystring)
-        return self.config_client.request(
-            constants.GET,
-            constants.GET_VCENTER_LIST_URL.format(self.server_ip),
-            querystring=querystring, all_pages=all_pages
-        )
+        vcenter_list = self.config_client.\
+            request(constants.GET,
+                    constants.GET_VCENTER_LIST_URL.format(self.server_ip),
+                    querystring=querystring, all_pages=all_pages)
+
+        if vcenter_list:
+            resp_list = []
+            for vcenter in vcenter_list:
+                resp_dict = self.get_vcenter_details(vcenter['id'])
+                resp_list.append(resp_dict)
+            return resp_list
+        return vcenter_list
 
     def get_vcenter_details(self, vcenter_id):
         """Get vcenter details.
@@ -847,12 +937,11 @@ class Configuration:
         """
         LOG.info("Getting vcenter details by ID: '%s'" % vcenter_id)
         querystring = constants.VCENTER_DETAILS_QUERY
-        if helpers.is_foot_hill_or_higher():
-            querystring = {
-                'select': 'id,instance_uuid,address,username,'
-                          'vendor_provider_status,'
-                          'vendor_provider_status_l10n'
-            }
+        if helpers.is_foot_hill_prime_or_higher():
+            querystring = constants.FHP_VCENTER_QUERY
+        elif helpers.is_foot_hill_or_higher():
+            querystring = constants.FHC_MALKA_VCENTER_QUERY
+
         return self.config_client.request(
             constants.GET,
             constants.GET_VCENTER_DETAILS_URL.format(self.server_ip,
@@ -861,20 +950,66 @@ class Configuration:
         )
 
     def modify_vcenter(self, vcenter_id, modify_param_dict):
-        """Register VASA provider.
+        """Modify vcenter attributes.
         :param vcenter_id: ID of the vcenter
         :type vcenter_id: str
-        :param modify_param_dict: Dict containing VASA provider credentials
+        :param modify_param_dict: Dict containing parameters for modification
         :type modify_param_dict: dict
         :return: Details of vcenter
         :rtype: dict
         """
-        LOG.info("Registering VASA provider: '%s'" % vcenter_id)
+        LOG.info("Modifying vCenter attributes: '%s'" % vcenter_id)
         self.config_client.request(constants.PATCH,
                                    constants.MODIFY_VCENTER_URL.format(
                                        self.server_ip, vcenter_id),
                                    payload=modify_param_dict)
         return self.get_vcenter_details(vcenter_id)
+
+    def add_vcenter(self, add_params):
+        """
+        Add a vcenter to the unified PowerStore model.
+        vcenter can not be added to unified+ deployment
+        :param add_params: the parameters to add vcenter
+        :type add_params:dict
+        :return: ID of the vcenter if addition is successful
+        :rtype: dict
+        """
+        LOG.info("Adding a vcenter.")
+
+        payload = dict()
+        if add_params:
+            for key, values in add_params.items():
+                payload[key] = values
+
+        return self.config_client.\
+            request(constants.POST,
+                    constants.ADD_VCENTER_URL.format(self.server_ip),
+                    payload=payload)
+
+    def remove_vcenter(self, vcenter_id, delete_vasa_provider=None):
+        """
+        Remove vcenter from Unified PowerStore model.
+        vcenter can not be removed from unified+ deployment
+        :param vcenter_id: ID of the vcenter
+        :type vcenter_id: str
+        :param delete_vasa_provider: whether to remove a VASA provider.
+                                     Removal will only happen if the provider
+                                     is not connected to any other PowerStore
+                                     system
+        :type delete_vasa_provider: bool
+        :return: None if success
+        :rtype: None
+        """
+        LOG.info("Removing vcenter: {0}.".format(vcenter_id))
+        payload = dict()
+        if delete_vasa_provider is not None:
+            payload['delete_vendor_provider'] = delete_vasa_provider
+
+        return self.config_client.\
+            request(constants.DELETE,
+                    constants.REMOVE_VCENTER_URL.format(self.server_ip,
+                                                        vcenter_id),
+                    payload=payload)
     # vCenter operations end
 
     # Appliance operations start
@@ -1283,7 +1418,7 @@ class Configuration:
     # smtp operations start
 
     def get_smtp_configs(self, filter_dict=None, all_pages=None):
-        """Get all SMTP configurationss.
+        """Get all SMTP configurations.
 
         :param filter_dict: (optional) Filter details
         :type filter_dict: dict
@@ -1606,7 +1741,7 @@ class Configuration:
 
     # Remote Support operations end
 
-    # Remote Support conatct operations start
+    # Remote Support contact operations start
 
     def get_remote_support_contact_list(self, filter_dict=None, all_pages=None):
         """Get all remote support contacts available on array.
@@ -1686,7 +1821,312 @@ class Configuration:
                 payload=modify_parameters)
         raise Exception("Not supported for PowerStore versions less than 2.0.0.0")
 
-    # Remote Support conatct operations end
+    # Remote Support contact operations end
+
+    # LDAP Domain operations start
+
+    def get_ldap_domain_configuration_list(self, filter_dict=None, all_pages=None):
+        """Get all LDAP domain configurations available on array.
+
+        :param filter_dict: (optional) Filter details
+        :type filter_dict: dict
+        :param all_pages: (optional) Indicates whether to return all ldap
+                          domain configurations or not
+        :type all_pages: bool
+        :return: List of ldap domain configurations on array
+        :rtype: list[dict]
+        """
+        LOG.info(
+            "Getting all ldap domain with filter: '%s' and all_pages: '%s'"
+            % (filter_dict, all_pages))
+
+        if all_pages:
+            raise Exception("Pagination is not supported for LDAP domain.")
+        if not filter_dict:
+            querystring = helpers.prepare_querystring(
+                constants.LDAP_DOMAIN_DETAILS_QUERY)
+            return self.config_client.request(
+                constants.GET, constants.GET_LDAP_DOMAIN_LIST_URL.format(
+                    self.server_ip), querystring=querystring, all_pages=False)
+
+        resp = self.config_client.request(
+            constants.GET,
+            constants.GET_LDAP_DOMAIN_LIST_URL.format(self.server_ip),
+            querystring=constants.LDAP_DOMAIN_DETAILS_QUERY, all_pages=False)
+
+        filterable_keys = ['domain_name', 'id', 'protocol', 'ldap_server_type']
+        ldap_domain_resp = helpers.filtered_details(filterable_keys, filter_dict,
+                                                    resp, 'ldap_domain')
+
+        if ldap_domain_resp:
+            filter_list = []
+            ldap_domain_dict = self.get_ldap_domain_configuration_details(ldap_domain_resp[0]['id'])
+            filter_list.append(ldap_domain_dict)
+            return filter_list
+        return ldap_domain_resp
+
+    def get_ldap_domain_configuration_details(self, ldap_domain_id):
+        """ Get details of a ldap domain instance.
+
+        :param ldap_domain_id: Unique identifier of the ldap domain
+        :type ldap_domain_id: str
+        :return: ldap domain details
+        :rtype: dict
+        """
+        LOG.info("Getting ldap domain details by ID: '%s'" % ldap_domain_id)
+
+        return self.config_client.request(
+            constants.GET,
+            constants.GET_LDAP_DOMAIN_DETAILS_URL.format(
+                self.server_ip, ldap_domain_id),
+            querystring=constants.LDAP_DOMAIN_DETAILS_QUERY)
+
+    def get_ldap_domain_configuration_details_by_name(self, ldap_domain_name):
+        """ Get details of a ldap domain instance.
+
+        :param ldap_domain_name: LDAP domain name
+        :type ldap_domain_name: str
+        :return: ldap domain details
+        :rtype: dict
+        """
+        LOG.info("Getting ldap domain details by name")
+
+        resp = self.config_client.request(
+            constants.GET,
+            constants.GET_LDAP_DOMAIN_LIST_URL.format(
+                self.server_ip, ldap_domain_name),
+            querystring=helpers.prepare_querystring(
+                constants.LDAP_DOMAIN_DETAILS_QUERY,
+                name=constants.EQUALS + ldap_domain_name)
+        )
+
+        filterable_keys = ['domain_name', 'id', 'protocol', 'ldap_server_type']
+        filter_dict = {'domain_name': 'eq.{0}'.format(ldap_domain_name)}
+        resp = helpers.filtered_details(filterable_keys, filter_dict,
+                                        resp, 'ldap_domain')
+        if resp:
+            return self.get_ldap_domain_configuration_details(resp[0]['id'])
+
+    def create_ldap_domain_configuration(self, create_parameters):
+        """ Create LDAP domain configuration.
+
+        :param create_parameters: Parameters for creating LDAP domain
+        :type create_parameters: dict
+        :return: Unique identifier of the new LDAP domain instance created
+        :rtype: dict
+        """
+        LOG.info("creating LDAP domain")
+
+        return self.config_client.request(
+            constants.POST,
+            constants.CREATE_LDAP_DOMAIN_URL.format(
+                self.server_ip), payload=create_parameters)
+
+    def modify_ldap_domain_configuration(self, ldap_domain_id, modify_parameters):
+        """ Modify LDAP domain configuration.
+
+        :param ldap_domain_id: Unique ID of the LDAP domain instance
+        :type ldap_domain_id: str
+        :param modify_parameters: Parameters for modifying LDAP domain
+        :type modify_parameters: dict
+        :return: None
+        :rtype: None
+        """
+        LOG.info("Modifying LDAP domain configuration id: '%s'" % ldap_domain_id)
+
+        return self.config_client.request(
+            constants.PATCH,
+            constants.MODIFY_LDAP_DOMAIN_URL.format(
+                self.server_ip, ldap_domain_id), payload=modify_parameters)
+
+    def delete_ldap_domain_configuration(self, ldap_domain_id):
+        """ Delete LDAP domain configuration.
+
+        :param ldap_domain_id: Unique ID of the LDAP domain instance
+        :type ldap_domain_id: str
+        :return: None
+        :rtype: None
+        """
+        LOG.info("Deleting LDAP domain configuration id: '%s'" % ldap_domain_id)
+
+        return self.config_client.request(
+            constants.DELETE,
+            constants.DELETE_LDAP_DOMAIN_URL.format(
+                self.server_ip, ldap_domain_id))
+
+    def verify_ldap_domain_configuration(self, ldap_domain_id):
+        """ Verify LDAP domain configuration.
+
+        :param ldap_domain_id: Unique ID of the LDAP domain instance
+        :type ldap_domain_id: str
+        :return: None
+        :rtype: None
+        """
+        LOG.info("Verifying LDAP domain configuration id: '%s'" % ldap_domain_id)
+
+        return self.config_client.request(
+            constants.POST,
+            constants.VERIFY_LDAP_DOMAIN_URL.format(
+                self.server_ip, ldap_domain_id))
+
+    # LDAP Domain operations end
+
+    # LDAP Account operations begin
+
+    def get_ldap_account_list(self, filter_dict=None, all_pages=None):
+        """Get all LDAP accounts available on array.
+
+        :param filter_dict: (optional) Filter details
+        :type filter_dict: dict
+        :param all_pages: (optional) Indicates whether to return all LDAP
+                          accounts or not
+        :type all_pages: bool
+        :return: List of LDAP accounts on array
+        :rtype: list[dict]
+        """
+        LOG.info(
+            "Getting all ldap accounts with filter: '%s' and all_pages: '%s'"
+            % (filter_dict, all_pages))
+
+        if all_pages:
+            raise Exception("Pagination is not supported for LDAP accounts.")
+        if not filter_dict:
+            querystring = helpers.prepare_querystring(
+                constants.LDAP_ACCOUNT_DETAILS_QUERY)
+            return self.config_client.request(
+                constants.GET, constants.GET_LDAP_ACCOUNT_LIST_URL.format(
+                    self.server_ip), querystring=querystring, all_pages=False)
+
+        resp = self.config_client.request(
+            constants.GET,
+            constants.GET_LDAP_ACCOUNT_LIST_URL.format(self.server_ip),
+            querystring=constants.LDAP_ACCOUNT_DETAILS_QUERY, all_pages=False)
+
+        filterable_keys = ['name', 'id', 'role_id', 'type']
+        ldap_account_resp = helpers.filtered_details(filterable_keys, filter_dict,
+                                                     resp, 'ldap_accounts')
+
+        # Return all the details for each LDAP account
+        if ldap_account_resp:
+            resp_list = []
+            for resp in ldap_account_resp:
+                resp_dict = self.get_ldap_account_details(resp['id'])
+                resp_list.append(resp_dict)
+            return resp_list
+        return ldap_account_resp
+
+    def get_ldap_account_details(self, ldap_account_id):
+        """ Get details of a LDAP account instance.
+
+        :param ldap_account_id: Unique identifier of the LDAP Account
+        :type ldap_account_id: str
+        :return: LDAP account details
+        :rtype: dict
+        """
+        LOG.info("Getting LDAP account details by ID: '%s'" % ldap_account_id)
+
+        return self.config_client.request(
+            constants.GET,
+            constants.GET_LDAP_ACCOUNT_DETAILS_URL.format(
+                self.server_ip, ldap_account_id),
+            querystring=constants.LDAP_ACCOUNT_DETAILS_QUERY)
+
+    def get_ldap_account_details_by_name(self, ldap_account_name):
+        """ Get details of an ldap account instance.
+
+        :param ldap_account_name: LDAP account name
+        :type ldap_account_name: str
+        :return: ldap account details
+        :rtype: dict
+        """
+        LOG.info("Getting LDAP Account details by name: '%s'" % ldap_account_name)
+        resp = self.get_ldap_account_list()
+
+        for account in resp:
+            if account['name'] == ldap_account_name:
+                return self.get_ldap_account_details(account['id'])
+
+
+    def create_ldap_account(self, create_parameters):
+        """ Create LDAP account configuration.
+
+        :param create_parameters: Parameters for creating LDAP account
+        :type create_parameters: dict
+        :return: Unique identifier of the new LDAP account instance created
+        :rtype: dict
+        """
+        LOG.info("creating LDAP account")
+
+        return self.config_client.request(
+            constants.POST,
+            constants.CREATE_LDAP_ACCOUNT_URL.format(
+                self.server_ip), payload=create_parameters)
+
+    def modify_ldap_account_details(self, ldap_account_id, modify_parameters):
+        """ Modify LDAP account configuration.
+
+        :param ldap_account_id: Unique ID of the LDAP account instance
+        :type ldap_account_id: str
+        :param modify_parameters: Parameters for modifying LDAP account
+        :type modify_parameters: dict
+        :return: None
+        :rtype: None
+        """
+        LOG.info("Modifying LDAP account id: '%s'" % ldap_account_id)
+
+        return self.config_client.request(
+            constants.PATCH,
+            constants.MODIFY_LDAP_ACCOUNT_URL.format(
+                self.server_ip, ldap_account_id), payload=modify_parameters)
+
+    def delete_ldap_account(self, ldap_account_id):
+        """ Delete LDAP account.
+
+        :param ldap_account_id: Unique ID of the LDAP account instance
+        :type ldap_account_id: str
+        :return: None
+        :rtype: None
+        """
+        LOG.info("Deleting LDAP account configuration id: '%s'" % ldap_account_id)
+
+        return self.config_client.request(
+            constants.DELETE,
+            constants.DELETE_LDAP_ACCOUNT_URL.format(
+                self.server_ip, ldap_account_id))
+
+
+    # LDAP Account operations end
+
+    # Virtual volume operations begin
+
+    def get_virtual_volume_list(self, filter_dict=None, all_pages=None):
+        """Get all virtual volumes available on array.
+        :param filter_dict: (optional) Filter details
+        :type filter_dict: dict
+        :param all_pages: (optional) Indicates whether to return all
+                          virtual volumes or not
+        :type all_pages: bool
+        :return: List of virtual volumes on array
+        :rtype: list[dict]
+        """
+        LOG.info("Getting volumes with filter: '%s' and all_pages: %s"
+                 % (filter_dict, all_pages))
+        querystring = helpers.prepare_querystring(
+            constants.VIRTUAL_VOLUME_DETAILS_QUERY,
+            filter_dict)
+        if helpers.is_foot_hill_prime_or_higher():
+            querystring = helpers.prepare_querystring(
+            constants.VIRTUAL_VOLUME_FHP_DETAILS_QUERY,
+            filter_dict)
+        LOG.info("Querystring: '%s'" % querystring)
+        return self.config_client.request(constants.GET,
+                                   constants.GET_VIRTUAL_VOLUME_LIST_URL.format
+                                   (self.server_ip), payload=None,
+                                   querystring=querystring,
+                                   all_pages=all_pages)
+
+    # Virtual volume operations end
+
 
     @staticmethod
     def _prepare_local_user_payload(**kwargs):
@@ -1714,3 +2154,22 @@ class Configuration:
             if kwargs.get(argname) is not None:
                 payload[argname] = kwargs[argname]
         return payload
+
+    @staticmethod
+    def _prepare_create_cluster_payload(is_http_redirect_enabled, **kwargs):
+        """Prepare payload for creation of cluster
+        :return: Request body
+        :rtype: dict
+        """
+        payload = dict()
+        for agrname in ('cluster', 'appliances', 'dns_servers', 'ntp_servers',
+                        'physical_switches', 'networks', 'vcenters'):
+            if kwargs.get(agrname) is not None:
+                payload[agrname] = kwargs[agrname]
+
+        if is_http_redirect_enabled is not None:
+            security_config = dict()
+            security_config['is_http_redirect_enabled'] = is_http_redirect_enabled
+            payload['security_config'] = security_config
+
+        return payload
