@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright: (c) 2019, Dell EMC
+# Copyright: (c) 2019, Dell Technologies
 
 """Module for PowerStore constants"""
 
@@ -46,20 +46,50 @@ FHC_VOLUME_DETAILS_QUERY = {
               "node_affinity,node_affinity_l10n"
 }
 
+FHP_VOLUME_DETAILS_QUERY = {
+    "select": "id,name,description,type,wwn,appliance_id,state,size,"
+              "creation_timestamp,protection_policy_id,performance_policy_id,"
+              "protection_policy(name,id),performance_policy(name,id),"
+              "is_replication_destination,logical_used,"
+              "protection_data,location_history,type_l10n,state_l10n,"
+              "host_group(name,id),volume_groups(name,id),"
+              "mapped_volumes(id,logical_unit_number),nsid,nguid,"
+              "node_affinity,node_affinity_l10n,metro_replication_session_id,"
+              "is_host_access_available,app_type,app_type_other,app_type_l10n,"
+              "migration_session_id"
+}
+
 # Host Query
-SELECT_ALL_HOST = {"select": "id,name,description,os_type,"
-                             "host_group_id,"
-                             "host_initiators,os_type_l10n,"
-                             "mapped_hosts(id,logical_unit_number,"
-                             "host_group(id,name),volume(id,name))"
+SELECT_ALL_HOST = {
+    "select": "id,name,description,os_type,"
+              "host_group_id,"
+              "host_initiators,os_type_l10n,"
+              "mapped_hosts(id,logical_unit_number,"
+              "host_group(id,name),volume(id,name)),"
+              "host_virtual_volume_mappings(id,host_id,host_group_id,virtual_volume_id)"
                    }
 FHC_HOST_DETAILS_QUERY = {
     "select": "id,name,description,os_type,host_group_id,host_initiators,"
               "os_type_l10n,mapped_hosts(id,logical_unit_number,"
-              "host_group(id,name),volume(id,name)),type,type_l10n"
+              "host_group(id,name),volume(id,name)),type,type_l10n,"
+              "host_virtual_volume_mappings(id,host_id,host_group_id,virtual_volume_id)"
+}
+FHP_HOST_DETAILS_QUERY = {
+    "select": "id,name,description,type,os_type,host_group_id,"
+              "host_connectivity,os_type_l10n,"
+              "mapped_hosts(id,logical_unit_number,host_group(id,name),"
+              "volume(id,name)),type_l10n,host_connectivity_l10n,"
+              "initiators(id,port_name,port_type,chap_single_username,"
+              "chap_mutual_username,active_sessions),host_initiators,"
+              "host_virtual_volume_mappings(id,host_id,host_group_id,virtual_volume_id)"
 }
 
 SELECT_ALL_HOST_GROUP = {"select": "name,id,description,hosts(id,name)"}
+FHP_HOST_GROUP_QUERY = {"select": "name,id,description,hosts(id,name,host_virtual_volume_mappings(id,"
+                                  "host_id,host_group_id,virtual_volume_id)),"
+                                  "host_connectivity,host_connectivity_l10n,"
+                                  "mapped_host_groups(id,volume_id,"
+                                  "logical_unit_number)"}
 
 SELECT_ALL_VG = {"select": "id,name,description,creation_"
                            "timestamp, member_type,"
@@ -111,6 +141,57 @@ SELECT_ALL_FILESYSTEM = {"select": "id,name, description,"
                          "creator_type_l10n,nas_server(name,id),"
                          "protection_policy(name,id)"}
 
+SELECT_ALL_FILESYSTEM_PRIME = {"select": "id,name, description,"
+                               "parent_id, filesystem_type, size_total,size_used,"
+                               "access_policy,locking_policy,"
+                               "folder_rename_policy, is_smb_sync_writes_enabled,"
+                               "is_smb_op_locks_enabled, is_smb_no_notify_enabled,"
+                               "is_smb_notify_on_access_enabled,"
+                               "is_smb_notify_on_write_enabled,"
+                               "smb_notify_on_change_dir_depth,"
+                               "is_async_MTime_enabled, is_quota_enabled,"
+                               "grace_period, default_hard_limit,"
+                               "default_soft_limit, creation_timestamp,"
+                               "expiration_timestamp, last_refresh_timestamp,"
+                               "last_writable_timestamp, is_modified,access_type,"
+                               "creator_type, filesystem_type_l10n,"
+                               "access_policy_l10n, locking_policy_l10n,"
+                               "folder_rename_policy_l10n, access_type_l10n,"
+                               "creator_type_l10n,nas_server(name,id),"
+                               "protection_policy(name,id),"
+                               "file_events_publishing_mode,"
+                               "file_events_publishing_mode_l10n,"
+                               "config_type, config_type_l10n,flr_attributes,"
+                               "host_io_size,host_io_size_l10n"}
+
+FILESYSTEM_PRIME = ['config_type', 'is_async_MTime_enabled',
+                    'file_events_publishing_mode', 'flr_attributes',
+                    'host_io_size']
+
+FHP_NAS_QUERYSTRING = {"select": "id,name, description, operational_status,"
+                       "current_node_id,preferred_node_id,"
+                       "default_unix_user,default_windows_user,"
+                       "current_unix_directory_service,"
+                       "is_username_translation_enabled,"
+                       "is_auto_user_mapping_enabled,"
+                       "production_IPv4_interface_id,"
+                       "production_IPv6_interface_id,"
+                       "backup_IPv4_interface_id,"
+                       "backup_IPv6_interface_id,"
+                       "current_preferred_IPv4_interface_id,"
+                       "current_preferred_IPv6_interface_id,"
+                       "protection_policy_id,"
+                       "operational_status_l10n,"
+                       "current_unix_directory_service_l10n,"
+                       "file_interfaces(name,id,ip_address),"
+                       "nfs_servers,smb_servers,"
+                       "file_ldaps,file_nises,file_systems(id,name)"
+                       "file_events_publishing_mode,"
+                       "is_replication_destination,"
+                       "is_production_mode_enabled,"
+                       "current_unix_directory_service_l10n,"
+                       "file_events_publishing_mode_l10n"}
+                       
 SELECT_ALL_NAS_SERVER = {"select": "id,name, description, operational_status,"
                          "current_node_id,preferred_node_id,"
                          "default_unix_user,default_windows_user,"
@@ -242,6 +323,20 @@ IP_PORT_DETAILS_QUERY = {
 VCENTER_DETAILS_QUERY = {
     'select': 'id,instance_uuid,address,username'
 }
+FHC_MALKA_VCENTER_QUERY = {
+    'select': 'id,instance_uuid,address,username,vendor_provider_status,'
+              'vendor_provider_status_l10n,'
+              'virtual_machines(id,name,instance_uuid,type,status,'
+              'virtual_volumes,protection_policy_id)'
+}
+FHP_VCENTER_QUERY = {
+    'select': 'id,instance_uuid,address,username,version,'
+              'vendor_provider_status,vendor_provider_status_l10n,'
+              'virtual_machines(id,name,instance_uuid,type,status,'
+              'virtual_volumes,protection_policy_id),'
+              'datastores(id,instance_uuid,name,type,'
+              'storage_container_id),vsphere_hosts(id,name,version)'
+}
 
 # Appliance details
 APPLIANCE_DETAILS_QUERY = {
@@ -261,14 +356,27 @@ REMOTE_SYSTEM_DETAILS_QUERY = {
               'session_chap_mode_l10n,data_network_latency_l10n,'
               'import_sessions,replication_sessions'
 }
+REMOTE_SYSTEM_FHP_DETAILS_QUERY = {
+    'select': 'id,name,description,serial_number,version,management_address,'
+              'management_port,type,user_name,state,data_connection_type,'
+              'data_connection_state,iscsi_addresses,fc_target_wwns,'
+              'discovery_chap_mode,session_chap_mode,data_network_latency,'
+              'data_connections,capabilities,file_connection_address,'
+              'file_connection_state,vnx_file_username,import_sessions,'
+              'appliance_details,type_l10n,replication_sessions,'
+              'state_l10n,data_connection_type_l10n,file_connection_state_l10n,'
+              'data_connection_state_l10n,discovery_chap_mode_l10n,'
+              'session_chap_mode_l10n,data_network_latency_l10n,'
+              'capabilities_l10n,storage_container_destinations'
+}
 
 # Certificate details
 CERTIFICATE_DETAILS_QUERY = {
     'select': 'id,type,type_l10n,service,service_l10n,scope,is_current,'
-               'is_valid,members(subject,serial_number,signature_algorithm,'
-               'issuer,valid_from,valid_to,public_key_algorithm,key_length,'
-               'thumbprint_algorithm,thumbprint_algorithm_l10n,thumbprint,'
-               'certificate,depth,subject_alternative_names)'
+              'is_valid,members(subject,serial_number,signature_algorithm,'
+              'issuer,valid_from,valid_to,public_key_algorithm,key_length,'
+              'thumbprint_algorithm,thumbprint_algorithm_l10n,thumbprint,'
+              'certificate,depth,subject_alternative_names)'
 }
 
 # Security config details
@@ -307,6 +415,33 @@ REMOTE_SUPPORT_DETAILS_QUERY = {
 REMOTE_SUPPORT_CONTACT_DETAILS_QUERY = {
     'select': 'id,email,first_name,last_name,phone'
 }
+
+# LDAP Domain details
+LDAP_DOMAIN_DETAILS_QUERY = {
+    'select': 'id,domain_name,ldap_servers,port,ldap_server_type,protocol,bind_user,ldap_timeout,'
+              'is_global_catalog,user_id_attribute,user_object_class,user_search_path,'
+              'group_name_attribute,group_member_attribute,group_object_class,'
+              'group_search_path,group_search_level,ldap_server_type_l10n,protocol_l10n'
+}
+VIRTUAL_VOLUME_FHP_DETAILS_QUERY = {
+    'select': 'id,name,size,type,usage_type,appliance_id,storage_container_id,io_priority,profile_id,'
+              'replication_group_id,creator_type,is_readonly,migration_session_id,virtual_machine_uuid,'
+              'family_id,parent_id,source_id,source_timestamp,creation_timestamp,naa_name,'
+              'is_replication_destination,location_history,protection_policy_id,nsid,nguid,type_l10n,'
+              'usage_type_l10n,io_priority_l10n,creator_type_l10n,'
+              'host_virtual_volume_mappings(id,host_id,host_group_id,virtual_volume_id)'
+}
+VIRTUAL_VOLUME_DETAILS_QUERY = {
+    'select': 'id,name,size,type,usage_type,appliance_id,storage_container_id,io_priority,profile_id,'
+              'creator_type,is_readonly,migration_session_id,virtual_machine_uuid,'
+              'family_id,parent_id,source_id,source_timestamp,creation_timestamp,'
+              'location_history,type_l10n,usage_type_l10n,io_priority_l10n,creator_type_l10n,'
+              'host_virtual_volume_mappings(id,host_id,host_group_id,virtual_volume_id)'
+}
+# LDAP Account details
+LDAP_ACCOUNT_DETAILS_QUERY = {
+    'select': 'id,role_id,domain_id,name,type,type_l10n,dn'
+}
 # Select all Snapshot
 
 EQUALS = 'eq.'
@@ -327,6 +462,11 @@ UNMAP_VOLUME_FROM_HOST_URL = 'https://{0}/api/rest/volume/{1}/detach'
 RESTORE_VOLUME_FROM_SNAPSHOT_URL = 'https://{0}/api/rest/volume/{1}/restore'
 GET_VOLUME_BY_NAME_URL = VOLUME_CREATE_URL
 CREATE_VOLUME_SNAPSHOT_URL = 'https://{0}/api/rest/volume/{1}/snapshot'
+CLONE_VOLUME_URL = 'https://{0}/api/rest/volume/{1}/clone'
+REFRESH_VOLUME_URL = 'https://{0}/api/rest/volume/{1}/refresh'
+RESTORE_VOLUME_URL = 'https://{0}/api/rest/volume/{1}/restore'
+CONFIGURE_METRO_VOLUME = 'https://{0}/api/rest/volume/{1}/configure_metro'
+END_METRO_VOLUME = 'https://{0}/api/rest/volume/{1}/end_metro'
 
 # Host endpoints
 GET_HOST_LIST_URL = 'https://{0}/api/rest/host'
@@ -354,6 +494,9 @@ CREATE_VOLUME_GROUP_URL = GET_VOLUME_GROUP_LIST_URL
 GET_VOLUME_GROUP_DETAILS_URL = 'https://{0}/api/rest/volume_group/{1}'
 MODIFY_VOLUME_GROUP_URL = GET_VOLUME_GROUP_DETAILS_URL
 DELETE_VOLUME_GROUP_URL = GET_VOLUME_GROUP_DETAILS_URL
+CLONE_VOLUME_GROUP_URL = 'https://{0}/api/rest/volume_group/{1}/clone'
+REFRESH_VOLUME_GROUP_URL = 'https://{0}/api/rest/volume_group/{1}/refresh'
+RESTORE_VOLUME_GROUP_URL = 'https://{0}/api/rest/volume_group/{1}/restore'
 ADD_MEMBERS_TO_VOLUME_GROUP_URL = \
     'https://{0}/api/rest/volume_group/{1}/add_members'
 REMOVE_MEMBERS_FROM_VOLUME_GROUP_URL = \
@@ -385,6 +528,7 @@ REPLICATION_SESSION_PAUSE_URL = 'https://{0}/api/rest/replication_session/{1}/pa
 REPLICATION_SESSION_RESUME_URL = 'https://{0}/api/rest/replication_session/{1}/resume'
 REPLICATION_SESSION_FAILOVER_URL = 'https://{0}/api/rest/replication_session/{1}/failover'
 REPLICATION_SESSION_REPROTECT_URL = 'https://{0}/api/rest/replication_session/{1}/reprotect'
+MODIFY_REPLICATION_SESSION_URL = REPLICATION_SESSION_OBJECT_URL
 
 # Remote system endpoints
 GET_REMOTE_SYSTEM_LIST_URL = 'https://{0}/api/rest/remote_system'
@@ -392,6 +536,7 @@ GET_REMOTE_SYSTEM_DETAILS_URL = 'https://{0}/api/rest/remote_system/{1}'
 CREATE_REMOTE_SYSTEM_URL = GET_REMOTE_SYSTEM_LIST_URL
 MODIFY_REMOTE_SYSTEM_URL = GET_REMOTE_SYSTEM_DETAILS_URL
 DELETE_REMOTE_SYSTEM_URL = GET_REMOTE_SYSTEM_DETAILS_URL
+GET_REMOTE_APPLIANCE_URL = 'https://{0}/api/rest/remote_system/{1}/query_appliances'
 
 
 # Protection Policy endpoint
@@ -473,6 +618,8 @@ GET_IP_POOL_LIST_URL = 'https://{0}/api/rest/ip_pool_address'
 GET_CLUSTER_DETAILS_URL = 'https://{0}/api/rest/cluster/{1}'
 GET_CLUSTER_LIST_URL = 'https://{0}/api/rest/cluster'
 MODIFY_CLUSTER_URL = GET_CLUSTER_DETAILS_URL
+CREATE_CLUSTER_URL = GET_CLUSTER_LIST_URL
+CREATE_CLUSTER_VALIDATE_URL = 'https://{0}/api/rest/cluster/validate_create'
 
 # CHAP config endpoints
 GET_CHAP_CONFIG_LIST_URL = 'https://{0}/api/rest/chap_config'
@@ -505,7 +652,9 @@ GET_JOB_DETAILS_URL = 'https://{0}/api/rest/job/{1}'
 # vCenter endpoints
 GET_VCENTER_LIST_URL = 'https://{0}/api/rest/vcenter'
 GET_VCENTER_DETAILS_URL = 'https://{0}/api/rest/vcenter/{1}'
+ADD_VCENTER_URL = GET_VCENTER_LIST_URL
 MODIFY_VCENTER_URL = GET_VCENTER_DETAILS_URL
+REMOVE_VCENTER_URL = GET_VCENTER_DETAILS_URL
 
 # Appliance endpoints
 GET_APPLIANCE_LIST_URL = 'https://{0}/api/rest/appliance'
@@ -565,3 +714,22 @@ SEND_ALERT_REMOTE_SUPPORT_URL = 'https://{0}/api/rest/remote_support/{1}/send_te
 GET_REMOTE_SUPPORT_CONTACT_LIST_URL = 'https://{0}/api/rest/remote_support_contact'
 GET_REMOTE_SUPPORT_CONTACT_DETAILS_URL = 'https://{0}/api/rest/remote_support_contact/{1}'
 MODIFY_REMOTE_SUPPORT_CONTACT_URL = GET_REMOTE_SUPPORT_CONTACT_DETAILS_URL
+
+# LDAP Domain endpoints
+GET_LDAP_DOMAIN_LIST_URL = 'https://{0}/api/rest/ldap_domain'
+GET_LDAP_DOMAIN_DETAILS_URL = 'https://{0}/api/rest/ldap_domain/{1}'
+CREATE_LDAP_DOMAIN_URL = GET_LDAP_DOMAIN_LIST_URL
+MODIFY_LDAP_DOMAIN_URL = GET_LDAP_DOMAIN_DETAILS_URL
+DELETE_LDAP_DOMAIN_URL = GET_LDAP_DOMAIN_DETAILS_URL
+VERIFY_LDAP_DOMAIN_URL = 'https://{0}/api/rest/ldap_domain/{1}/verify'
+
+# LDAP Account endpoints
+GET_LDAP_ACCOUNT_LIST_URL = 'https://{0}/api/rest/ldap_account'
+GET_LDAP_ACCOUNT_DETAILS_URL = 'https://{0}/api/rest/ldap_account/{1}'
+CREATE_LDAP_ACCOUNT_URL = GET_LDAP_ACCOUNT_LIST_URL
+MODIFY_LDAP_ACCOUNT_URL = GET_LDAP_ACCOUNT_DETAILS_URL
+DELETE_LDAP_ACCOUNT_URL = GET_LDAP_ACCOUNT_DETAILS_URL
+
+# virtual volume endpoints
+GET_VIRTUAL_VOLUME_LIST_URL = 'https://{0}/api/rest/virtual_volume'
+GET_VIRTUAL_VOLUME_DETAILS_URL = 'https://{0}/api/rest/virtual_volume/{1}'
